<?php

namespace App\Http\Controllers;

use DB;
use DataTables;
use Carbon\Carbon;
use Aws\Common\Aws;
use App\Models\Domain;
use Aws\Ses\SesClient;
use App\Models\Campaign;
use App\Models\Template;
use App\Models\Subscriber;
use App\Models\EmailReport;
use App\Mail\SubscriberMail;
use Illuminate\Http\Request;
use App\Models\Publish_Email;
use App\Jobs\MakeEmailCronJob;
use Illuminate\Support\Facades\Log;
use App\Models\PublishMailByCron;
use Illuminate\Support\Facades\Mail;
use PhpOffice\PhpSpreadsheet\Exception;

class CampaignController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        
        // $campaign = Campaign::with(['domains:id,url','publishEmail:id,campaign_id,sent_status'])
        //             ->orderBy('created_at','desc')->get();
        // return view('campaign.index' , compact('campaign'))->with('no',1);
        if (request()->ajax()) {
            $data = Campaign::select('id','campaign','domain_id','subject','status','created_at')
                            ->orderBy('id','desc');

            return Datatables::eloquent($data)
                    ->addIndexColumn()
                    ->escapeColumns([])
                    ->editColumn('id', function ($data) {
                        return $data->id; 
                    })
                    // ->editColumn('campaign', function($data){
                    //     return $data->campaign;
                    // })
                    ->addColumn('subject', function ($data) {
                    
                         return ($data->subject);
                    })
                    ->addColumn('domains', function ($data) {
                    
                        return ($data->domains->url);
                    })
                   ->addColumn('created_at', function ($data) {
                    
                    return (\Carbon\Carbon::parse($data->created_at)->format("d/m/Y"));
                    })
                    ->addColumn('publish_Email', function ($data) {
                    
                        return $data->sentEmailCount();
                    })
                   ->addColumn('sent_email', function ($data) {
                    
                    return ($data->inProcessEmailCount() );
                    })
                    ->addColumn('not_Sentemail', function ($data) {
                    
                        return ($data->remainingEmailCount() );
                    })
                    ->addColumn('status', function ($data) {
                    
                        if($data->status==0 )
                        {
                            return 'Initial';
                        }
                        elseif($data->status == 1)
                        {
                            return 'Start';
                        }
                        else{
                            return 'End';
                        }
                        
                    })
                    ->addColumn('action', 'campaign.campaign_action')
                    ->rawColumns(['action'])
                    ->toJson();
        }
        return view('campaign.index' );

        /*$campaign = Campaign::with(['domains:id,url','publishEmail:id,campaign_id,sent_status'])
                    ->orderBy('created_at','desc')->get();*/
                    
        // $campaign = Campaign::orderBy('id','DESC')->paginate(50);            
        // return view('campaign.index' , compact('campaign'))->with('no',1);
        // if (request()->ajax()) {
        //     $data = Campaign::select('campaign','email','gender','desig','address')->orderBy('id','desc');
        //     return Datatables::eloquent($data)
        //             ->addIndexColumn()
        //             ->addColumn('action', 'backend.user.user_action')
        //             ->rawColumns(['action'])
        //             ->toJson();
        // }
        // return view('campaign.index' );
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
        $domain = Domain::select('id','url')->get();
        $template = Template::select('id','title')->get();
        return view('campaign.add' , compact('domain','template'))->with('no',1);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        set_time_limit(120); // 2 minutes
        //
        $validate = $this->validate($request,[
                'domain' => 'required',
                'campaign' => 'required',
                'templates' => 'required',
                'subject' =>'required',
               
                
        ]);
        
        DB::beginTransaction();
        try{
            $campaign = new Campaign;
            $campaign->domain_id = $request->domain;
            $campaign->campaign = $request->campaign;
            $campaign->template_id = $request->templates;
            $campaign->status = '0';
            $campaign->subject = $request->subject;

            $campaign->save();
            
            $this->insertCampaignSuscribers($campaign->id);
            DB::commit();
            return redirect('/campaign')->with('success','Campaign added Successfully');
        }catch(Exception $e){
            DB::rollBack();
            // \Log::info("Error on start campaign - ".$e->getMessage()); // put this error on log file
            return redirect()->back()->with('error',$e->getMessage());
        }
       
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
        $domain = Domain::all();
        $template = Template::select('id','title')->get();
        $campaign = Campaign::find($id);
        return view('campaign/update', compact('domain','template','campaign'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
       

        $validate = $request->validate([
            'domain' => 'required',
            'templates' => 'required',
            'status' => 'required'
        ]);

        
        try{
           // DB::beginTransaction();
            
            $campaign =  Campaign::findOrFail($id);

            $campaign->campaign = $request->campaign;
            $campaign->template_id = $request->templates;
            $campaign->status = $request->status;
            $campaign->subject = $request->subject;
            $old_domain_id = $campaign->domain_id;
          
            if($request->domain != $campaign->domain_id)
            {
               $publishDel = Publish_Email::where('campaign_id',$id)->delete();
            }
             $campaign->domain_id = $request->domain;
             $campaign->update();

             if($old_domain_id != $campaign->domain_id){
                $this->insertCampaignSuscribers($campaign->id);
             }

           
           // DB::commit();
            return redirect('/campaign')->with('success', 'Campaign Updated is  successfully ');
        }catch(Exception $e){
            DB::rollBack();
            // \Log::info("Error on start campaign - ".$e->getMessage()); // put this error on log file
            return redirect()->back()->with('error',$e->getMessage());
        }
       
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroyCampaign($id)
    {
        try{
            if(request()->ajax()){
                $result = Campaign::where('id',$id)->delete();
                $publish = Publish_Email::where('campaign_id',$id)->delete();
                if($result && $publish){
                   return Response()->json(['message' => 'Campaign Deleted! Successfully','status'=>'200']);  
                }else{
                  return Response()->json(['message' => 'Campaign Not Deleted!','status'=>'201']);
                }
            }
            // $campaign = Campaign::findOrFail($id);
            //deleteing Publish Email Table
            // Publish_Email::where('campaign_id',$id)->delete();
            // $campaign->delete();
            // return redirect('/campaign')->with('success', 'Campaign is successfully Delete');
        }catch(Exception $e){
            return redirect('/campaign')->with('error', $e->getMessage());
        }
        
    }
    /*public function Publish_Email($id)
    {  
        $campaign = Campaign::findOrFail($id);
        $subscribers= $campaign->domains->activeSubscriberIds;
        //check subscribers 
        if(empty($subscribers) || count($subscribers) < 1 ){
            return redirect()->back()->with('error','Subscribers not found');
        }
        $no = 1;    
        $batchInsertArray = array(); // declare a blank array
        try{
            DB::beginTransaction();
            foreach($subscribers as $subscriber )
            {
                //creating a batch of 5k records for insert in publish email table
                $batchInsertArray[] = [
                    "campaign_id"=>$campaign->id,
                    "subscriber_id"=>$subscriber->id,
                    "sent_status"=>0, // initial status
                    "createdon"=>Carbon::now(),
                    "updatedon"=>Carbon::now(),
                ];
                if($no==5000){
                    //insert batch record on table
                    Publish_Email::insert($batchInsertArray);
                    $no = 0;
                    $batchInsertArray = array(); // redeclare a blank array
                }
                $no++;
            }
            //insert remaining subscriber of last batch
            if(count($batchInsertArray) > 0){
                Publish_Email::insert($batchInsertArray);
            }
            
            $campaigns = Campaign::find($id);
            $campaigns->status = 1;
            $campaigns->update();
            DB::commit();

            return redirect('/campaign')->with('success','Campaign was Started');
        }catch(Exception $e){
            DB::rollBack();
            \Log::info("Error on start campaign - ".$e->getMessage()); // put this error on log file
            return redirect()->back()->with('error',$e->getMessage());
        }
    }*/
    /*public function Email_Sent($id)
    {
        $campaign_id = $id;
        $campaign = Campaign::find($id);
        
        $subscriber= $campaign->domains->activeSubscribers;
        $publish = $campaign->publish;
        // dd($publish);
        

        Subscriber::where('domain_id', $campaign->domain_id)->chunk(200, function($subscriber) use ($campaign, $publish)
        {
            foreach($subscriber as $subscriber_ids )
            {
                $content = array(
            
                    'message'   =>   $campaign->template->template,
                    'subjects'  =>   $campaign->subject,
                );
                Mail::to($subscriber_ids->email)->send(new SubscriberMail($content));
                foreach($publish as $key)
                {
                    
                    $publishs = Publish_Email::find($key->id);
                    $publishs->sent_status = 1;
                    $publishs->update();
                }
                $campaigns = Campaign::find($campaign->id);
                $campaigns->status = 2;
                $campaigns->update();
            }
        });
        

        return redirect('/campaign')->with('success','Mail Sent Succesfully');
    }*/

   // publish mail function is running
   public function sendDataToMailByCron($campaign_id){
       
     // $campaign = Campaign::with(['domains:id,domain_email,email_limit','publishEmail.subscribers:id,name,email','template'])->where('id',$campaign_id)->first();
     $campaign = Campaign::with(['domains:id,domain_email,email_limit,mail_driver','publishEmail'=> function ($query) {
             $query->select('id','campaign_id','subscriber_id','sent_status')->where('sent_status','0');
          },'publishEmail.subscribers:id,name,email','template'])->where('id',$campaign_id)->first();
    //dd($campaign); 
        try{
            $email_limit_set = 2000;
            if($campaign->domains->email_limit){
                $email_limit_set = $campaign->domains->email_limit;
            }
  
            // query add in avariable
            $query = $campaign->publishEmail->take($email_limit_set);
     
      
           //here to data is not empty
           if(count($query) == 0 || is_null($query) || empty($query) ){
              return redirect('/campaign')->with('error','Publish Email Not Found ');
           }
       
           foreach($query->chunk(50) as  $publish){
              $sendData = $jobData = array();
              $sentStatusUpdateId = [];
                foreach($publish as $publishEmail){
                    $url = route('unsubscribe',base64_encode($publishEmail->subscribers->id));
                    $fullUrl = "<a href='".$url."'>unsubscribe</a>";
                    $mail_body = str_replace('unsubscribe_link', $fullUrl, $campaign->template->template);
                    
                    $sentStatusUpdateId[] =$publishEmail->id;
                    $sendData[] = [
                        'tbl_pub_ema_id'=> $publishEmail->id,
                        'from_emailid' =>$campaign->domains->domain_email,
                        'to_name' =>$publishEmail->subscribers->name,
                        'to_emailid' =>$publishEmail->subscribers->email,
                        'subject' =>$campaign->subject,
                        'body' => $mail_body
                    ];
                    $jobData[] = [
                        'tbl_pub_ema_id'=> $publishEmail->id,
                        'from_emailid' =>$campaign->domains->domain_email,
                        'to_name' =>$publishEmail->subscribers->name,
                        'to_emailid' =>$publishEmail->subscribers->email,
                        'subject' =>$campaign->subject,
                        'body' => $mail_body,
                        'mail_driver'=>$campaign->domains->mail_driver
                    ];
                }
        
                PublishMailByCron::insert($sendData);
                //dispatch(new MakeEmailCronJob($jobData))->delay(now()->addMinutes(1));
                dispatch(new MakeEmailCronJob($jobData));
                Publish_Email::whereIn('id',$sentStatusUpdateId)->update(['sent_status'=>config('constant.PUBLISH_EMAIL_IN_PROCESS')]); // 2= in process
            }
      
        }catch(Exception $ex){
      
            return redirect()->back()->with('error',$ex->getMessage());
        }

        return redirect('/campaign')->with('success','Campaign was Publish');
   }

    function insertCampaignSuscribers($campaign_id){
        //$campaign = Campaign::find($campaign_id);
        $campaign = Campaign::with(['domains.subscribers' => function($query){ $query->select('id','domain_id')->where('is_active',1);}])->where('id',$campaign_id)->first();
        //Publish Email Store Code.
        $subscribers = $campaign->domains->subscribers;
        //dd($subscribers);
      
        //check subscribers 
        if(empty($subscribers) || count($subscribers) < 1 ){
            return redirect()->back()->with('error','Subscribers not found');
        }
        foreach($subscribers->chunk(1000) as $subscriberData ){    
           // declare a blank array
            $batchInsertArray = array(); 
            foreach($subscriberData as $subscriber){
               //creating a batch of 5k records for insert in publish email table
                $batchInsertArray[] = [
                    "campaign_id"=>$campaign->id,
                    "domain_id" => $campaign->domain_id,
                    "subscriber_id"=> $subscriber->id,
                    "sent_status"=> config('constant.PUBLISH_EMAIL_INITIAL'),
                    "createdon"=> Carbon::now(),
                    "updatedon"=> Carbon::now(),
                    
                ];
            }
           
            //insert batch record on table
            Publish_Email::insert($batchInsertArray);
        }

        
    }



    
    /*old code of this
    private function insertCampaignSuscribers($campaign_id)
    {
        $campaign = Campaign::find($campaign_id);
         //Publish Email Store Code.
        $subscribers= $campaign->domains->activeSubscriberIds;
        //check subscribers 
        if(empty($subscribers) || count($subscribers) < 1 ){
            return redirect()->back()->with('error','Subscribers not found');
        }
        $no = 1;    
        $batchInsertArray = array(); // declare a blank array
         
            //  DB::beginTransaction();
            foreach($subscribers as $subscriber )
            {
                 //creating a batch of 5k records for insert in publish email table
                $batchInsertArray[] = [
                    "campaign_id"=>$campaign->id,
                    "domain_id" => $campaign->domain_id,
                    "subscriber_id"=>$subscriber->id,
                    "sent_status"=>0, // initial status
                    "createdon"=>Carbon::now(),
                    "updatedon"=>Carbon::now(),
                    
                ];
                if($no==5000){
                     //insert batch record on table
                    Publish_Email::insert($batchInsertArray);
                    $no = 0;
                    $batchInsertArray = array(); // redeclare a blank array
                }
                $no++;
            }
             //insert remaining subscriber of last batch
            if(count($batchInsertArray) > 0){
                Publish_Email::insert($batchInsertArray);
            }
             
            // $campaigns = Campaign::find($campaign_id);
            // $campaigns->status = 1;
            // $campaigns->update();

    }
            //  DB::commit();
 
        //      return redirect('/campaign')->with('success','Campaign was Started');
        //  }catch(Exception $e){
        //     //  DB::rollBack();
        //      \Log::info("Error on start campaign - ".$e->getMessage()); // put this error on log file
        //      return redirect()->back()->with('error',$e->getMessage());
        //  }

        
    
    
     * Save campaign email open history status.
      * @param  Str base64 encoded string $subscriber_id
      * @return \Illuminate\Http\Response
      */
    public function saveCampaignEmailOpenedRecord($publish_email_id){
        
        $publish_email_id = base64_decode($publish_email_id);
        try{
            //verify publish email exist or not
            $publish_email = Publish_Email::findOrFail($publish_email_id);
          
            $publish_email_report = EmailReport::updateOrCreate(['publish_email_id'=>$publish_email_id],
                [
                 'subscriber_id'=>$publish_email->subscriber_id,
                 'domain_id'=>$publish_email->domain_id,
                 'open_count'=>0,
                 'last_open'=>Carbon::now(),
            ]);

            
            
           /* if(empty($publish_email_report)){
                $publish_email_report = new EmailReport;
                $publish_email_report->publish_email_id = $publish_email->id;
                $publish_email_report->subscriber_id = $publish_email->subscriber_id;
                $publish_email_report->domain_id = $publish_email->subscribers->domain_id;
                $publish_email_report->open_count = 1;
                $publish_email_report->created_at = Carbon::now();
            }else{
                $publish_email_report->open_count = $publish_email_report->open_count+1;
            }

            $publish_email_report->last_open = Carbon::now(); 
            $publish_email_report->updated_at = Carbon::now(); 
            
            $publish_email_report->save();
            */ 
            return true;
        }catch(Exception $e){
            return false;
            //dd($e->getMessage());
        }
    }

    public function createCampaign(Campaign $campaign)

    {
        // dd($campaign->id);
        return view('templates.test_campaign',compact('campaign'));
    }

    // public function test(Request $request,$campaign_id)
    // {
    //     // dd($request->email);
    //     $test_campaign = Campaign::with('template')->findOrFail($campaign_id);

    //     $templates =['templates'=>$test_campaign->template->template,];
    //     $user['to'] = $request->email;
    //     $subject['subject'] = $test_campaign->subject;

    //     mail::send('templates.templateview',$templates,function($message) use ($user,$subject){
    //         $message->to($user['to']);
    //         $message->subject($subject['subject']);

    //     });

    // }
    public function test(Campaign $campaign, Request $request)
    {
        //  dd($campaign->template->template);
        // $test_campaign = Campaign::with('template')->findOrFail($campaign_id);
        $manish = $this->send_mail('sparkpost','editor@wjpps.com','phpom41@gmail.com','vishal','manish');
        echo "Vishal Sir"; die;
        $templates =['templates'=>$campaign->template->template,];
        $user['to'] = $request->email;
        $subject['subject'] = $campaign->subject;

        mail::send('templates.templateview',$templates,function($message) use ($user,$subject){
            $message->to($user['to']);
            $message->subject($subject['subject']);

        });

    }

    private function send_mail($driver,$mail_from,$mail_to,$body,$subject,$name=''){
        try{
            
            $site_name = "";
            Mail::mailer($driver)->send([],[],function($message)use($mail_to,$subject,$name,$mail_from,$site_name,$body){
                $message->to($mail_to,$name)->subject($subject);
                $message->from($mail_from,$site_name);
                $message->setBody($body, 'text/html');
            });
         
            return true;
            
        }catch(Exception $e){
            return false;
        }
    }





}