<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Imports\ContactImport;
use App\Imports\UserImport;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Bus;
use App\Jobs\ContactCsvProcess;
use Exception;

use App\Models\Domain;
use App\Models\ImportSubscriberDetail;

class ExcelImportController extends Controller
{
    public function contactImportCreate(){
        $domains = Domain::select('id','description')->orderBy('id','asc')->get();
        return view('excel_file_upload',compact('domains'));
    }



    public function contactImportStore(Request $request){
       

       $this->validate($request,[
          'domain'=>['required'],
          'import_file' =>['required','file']
       ]);
       
        


      try{
            
          $file = $request->file('import_file');
        
          $fileName = $request->file('import_file')->getClientOriginalName();

          //set prefix in file as date 
          $fileName = date('Y-m-d').'_'.$fileName;

              // file upload in public folder
          $file =$request->file('import_file')->move('uploads/import/contact/',$fileName);
          ini_set('max_execution_time', 300);
      
          // domain detail get
          $domain = Domain::findOrFail($request->domain);

          $import = new ContactImport($domain->id);
   
          $import->import($file);

          // set data to store in ImportSubscriberDetail table
          $import_details =[
            'domain_id'=> $domain->id,
            'domain_email'=>$domain->domain_email,
            'domain_desciption'=>$domain->description,
            'file_name' =>  $fileName,
            'total_insert_row' => $import->total_insert,
            'total_failure_row' => count($import->failures()), 
          
          ];
          // import file and data details save in ImportSubscriberDetail table

          ImportSubscriberDetail::create($import_details);

          // failures record to show
        
          if($import->failures()->isNotEmpty()){
            return back()->withFailures($import->failures());
          }

      }catch(\Maatwebsite\Excel\Validators\ValidationException $exception){

         return back()->withFailures($exception->failures());

      }catch(\Illuminate\Database\QueryException $exception){

         return back()->withErrors($exception->getMessage());

      }catch(Exception $e){
       
         return back()->withErrors($e->getMessage());
      } 

        

        return back()->withStatus('Excel File Imported Successfully');
    }



    public function emailCsvStore(Request $request){
       $this->validate($request,[
          'domain'=>['required'],
          'import_file' =>['required','file']
       ]);

       $domainId = $request->domain;


      if ($request->file('import_file')) {
          //file name
          $fileName = date('Ymd_His').'_'.$request->file('import_file')->getClientOriginalName();
          
            $data   =   file($request->import_file);
            // Chunking file
            $chunks = array_chunk($data, 1000);
        
            $header = ['email','name','domain_id'];
        
            $batch  = Bus::batch([])->name($fileName)->dispatch();

            foreach ($chunks as $key => $chunk) {
                $data = array_map('str_getcsv', $chunk);
               if ($key === 0) {
                
                  unset($data[0]);
                }
                            
                $newData =[];
                foreach($data as $key=> $value){
                 $newData[]= [$value[14],($value)?$value[0]:$value[2],$domainId];
                }
              

                $batch->add(new ContactCsvProcess($newData,$domainId,$header));
               
                 
            }
            return back()->withStatus('Excel File on Queue');
            //return $batch;
        }

        return 'please upload file';
    }

}
