<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Domain;
use App\Models\Subscriber;
use Carbon\Carbon;
use DataTables;

class SubscribersController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        //
        // $subscriber = Subscriber::orderBy('id','desc')->first();
        // dd($subscriber->domains->url);
        
        if (request()->ajax()) {
            // dd($request->domain);
            $subsQuery = Subscriber::query();
            $domain_id = $request->domain;
            if($domain_id)
            {
                $subsQuery->where('domain_id',$domain_id);

            }
           
            $data = $subsQuery->select('id','name','email','is_active','domain_id')->where('is_active',1)->orderBy('id','desc');
                
                
                return Datatables::of($data)
                        // ->make(true)
                        ->addIndexColumn()
                        ->escapeColumns([])
                        ->editColumn('id', function ($data) {
                            return $data->id; 
                        })
                        ->editColumn('name', function($data){
                            return isset($data->name) ? $data->name : 'NA';
                        })
                        ->addColumn('domains', function ($data) {
                        
                             return ($data->domains->url);
                        
                        })
                        
                        ->addColumn('action', 'subscribers.subscriber_action')
                        ->rawColumns(['action'])
                        ->editColumn('is_active', function($data){
                            $Active=$Deactive=$Unsubscribed="";
                            switch ($data->is_active) {
                                  case "1":
                                    $Active ="selected";
                                    break;
                                  case "0":
                                   $Deactive ="selected";
                                    break;
                                  case "2":
                                    $Unsubscribed ="selected";
                                    break;
                                
                                }
                              switch ($data->is_active) {
                                  case "1":
                                    $is_active ='<strong id="stset" class="badge badge-success">'.$data->is_active.'</strong>';
                                    break;
                                  case "0":
                                   $is_active ='<strong id="stset" class="badge badge-warning">'.$data->is_active.'</strong>';
                                    break;
                                  case "2":
                                    $is_active ='<strong id="stset" class="badge badge-secondary">'.$data->is_active.'</strong>';
                                    break;
                                }
    
                            return $is_active.'<br><div class"row"><select id="select-is_active" data-id="'.$data->id.'">
                                    <option value="1" '.$Active.' >Active</option>
                                    <option value="0" '.$Deactive.'>Unactive
                                    </option>
                                    <option value="2" '. $Unsubscribed.' >Unsubscribed</option>
                                </select>                                            </div>';
                        }) 
                        ->make(true);
                        // ->toJson();
            }
            // $profile = User::all();
            $domain = Domain::select('id','url')->get();

            return view('subscribers/index',compact('domain'));
        // return view('subscribers/index',compact('subscriber'))->with('no',1);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
        $domain = Domain::select('id','url')->get();
        return view('subscribers/create',compact('domain'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
        $validate = $request->validate([
            'name' => 'required',
            'email' => 'required|unique:subscribers|regex:/(.+)@(.+)\.(.+)/i',
        ]);

        $subscriber = new Subscriber();
        $subscriber->name = $request->name;
        $subscriber->email = $request->email;
        $subscriber->domain_id = $request->domain;
        $subscriber->save();

        return redirect('/subscribers')->with('success', 'Subscriber Added is  successfully ');

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
        
        $subscriber = Subscriber::find($id);
        $domain = Domain::select('id','url')->get();
        return view('subscribers/update',compact('subscriber','domain'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
        $validate = $request->validate([
            'name' => 'required',
            'email' => 'required|regex:/(.+)@(.+)\.(.+)/i',
        ]);
        
        $subscriber = Subscriber::find($id);
        $subscriber->name = $request->name;
        $subscriber->email = $request->email;
        $subscriber->domain_id = $request->domain;
        $subscriber->update();

        return redirect('/subscribers')->with('success', 'Subscriber Updated is  successfully ');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroySubscriber($id)
    {
    
        if(request()->ajax()){
            $result = Subscriber::where('id',$id)->delete();
            if($result){
               return Response()->json(['message' => 'Subscriber Deleted! Successfully','status'=>'200']);  
            }else{
              return Response()->json(['message' => 'Subscriber Not Deleted!','status'=>'201']);
            }
        }
    }

    public function statusUpdate(Request $request,$id)
    {
    
        $data = Subscriber::findOrFail($id);

        $data->is_active = $request->is_active;
        $result = $data->update();
     
        if($result){
            return Response()->json(['message'=>'Subscriber status change Successfully','status'=>'200']);  
         }else{
           return Response()->json(['message'=>'Subscriber status not changed!','status'=>'201']);
         }
    }
    /**
     * Update subscriber status as unsubscrib.
     * @param  Str base64 encoded string $subscriber_id
     * @return \Illuminate\Http\Response
     */
    public function unsubscribeForSendEmail($subscriber_id){
        $subscriber_id = base64_decode($subscriber_id);
        $response = array();
        try{
            $subscriber = Subscriber::findOrFail($subscriber_id);
            $subscriber->is_active = config('constant.SUBSCRIBER_STATUS_UNSUBSCRIBE');
            $subscriber->updated = Carbon::now();
            $subscriber->save();
            $response['success'] = "You have unsubsribe successfully.";
        }catch(Exception $e){
            $response['error'] = "Oops! Internal server error.";
        }
        
        return view('unsubscribe-response',compact('response'));
    }
}
