<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\User;
use App\Models\Domain;
use App\Models\Domain_User;
use DataTables;



use Illuminate\Support\Facades\Hash;


class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    { 
        //
    
        if (request()->ajax()) {
        $data = User::select('id','name','email')->orderBy('id','desc');

            return Datatables::of($data)
                    ->addIndexColumn()
                    ->addColumn('action', 'user.user_action')
                    ->rawColumns(['action'])
                    ->toJson();
        }
        // $profile = User::all();
        return view('user/index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //

        $domainIds = Domain_user::pluck('domain_id')->toArray();
        $domain = Domain::whereNotIn('id',$domainIds)->select('id','url')->get();
        
 
        return view('user/create',compact('domain'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
        $validate = $request->validate([
            'name' => 'required',
            'password' => 'required|min:8',
            'email' => 'required|unique:users|regex:/(.+)@(.+)\.(.+)/i',
        ]);

        $user = new User();
        $user->name = $request->name;
        $user->email = $request->email;
        $user->password = Hash::make('$request->password');
        $user->save();

        
        $domain_ids = $request->domain;
        

        foreach($domain_ids as $domaine )
        {
            $domain = new Domain_user();
            $domain->user_id = $user->id;
            $domain->domain_id = $domaine;
            $domain->save();    
        }
        

        return redirect('/user')->with('success', 'User Added is  successfully ');


    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
        $user = User::find($id);
        $userIds = Domain_user::pluck('user_id')->toArray();
        $domain = Domain::whereIn('id',$userIds)->select('id','url')->get();
        return view('user/update',compact('user','domain'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
        $validate = $request->validate([
            'name' => 'required',
            
            'email' => 'required|regex:/(.+)@(.+)\.(.+)/i',
            
        ]);
        
        $user = User::find($id);
        Domain_user::where('user_id',$id)->delete();
        
        $user->name = $request->name;
        $user->email = $request->email;
        if ($request->has('password') && $request->password != '')
        {
            
            $user->password = Hash::make('$request->password');
        }
        $domain_ids = $request->domain;
        foreach($domain_ids as $domaine )
        {
            $domain = new Domain_user();
            $domain->user_id = $id;
            $domain->domain_id = $domaine;
            $domain->save();    
        }
        $user->update();
        return redirect('/user')->with('success', 'User Updated is  successfully ');

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroyUser($id)
    {
        
        if(request()->ajax()){
            $result = User::where('id',$id)->delete();
            if($result){
               return Response()->json(['message' => 'User Deleted! Successfully','status'=>'200']);  
            }else{
              return Response()->json(['message' => 'User Not Deleted!','status'=>'201']);
            }
        }
        
    }
}
